@@ -1,8 +1,13 @@
-from SimpleXMLRPCServer import SimpleXMLRPCServer
 from datetime import datetime, timedelta
-from threading import Thread
+from threading import Thread, Event
 import re
-import simplejson
+
+try:
+    # Python 2.6+
+    import json
+except ImportError:
+    import simplejson as json
+    
 import socket
 import sys
 import time
@@ -10,87 +15,62 @@ import urllib
 import urllib2
 import xmlrpclib
 
-APP_TITLE = 'Clip2Zeus'
-DELIM = ' \n\r<>"\''
-URL_RE = re.compile('((\w+)://([^/%%%s]+)(/?[^%s]*))' % (DELIM, DELIM), re.I | re.M)
-INVALID_DOMAINS = ('2ze.us', 'bit.ly', 'tinyurl.com', 'tr.im', 'is.gd')
+from clip2zeus.globals import *
+from clip2zeus.config import *
 
-HEARTBEAT_INT = 30 # Interval to ensure we have a connection to 2ze.us (seconds)
-TIMEOUT_SEC = 10 # Number of seconds to wait on 2ze.us before giving up
-DEFAULT_PORT = 14694
-
-class UnsupportedPlatformError(StandardError): pass
-
-class Monitor(Thread):
-    """Regularly checks the system clipboard for data"""
-
-    def __init__(self, *args, **kwargs):
-        self.app = kwargs.pop('app')
-        super(Monitor, self).__init__(*args, **kwargs)
-
-    def run(self):
-        self._continue_processing = True
-
-        while self._continue_processing:
-            # don't do anything if the user only wants manual shortening
-            if self.app.interval <= 0:
-                time.sleep(1)
-                continue
-
-            try:
-                self.app.shorten_urls()
-                time.sleep(self.app.interval)
-            except KeyboardInterrupt:
-                self.app.quit()
-
-    def close(self):
-        self._continue_processing = False
+__author__ = 'Josh VanderLinden'
+__version__ = '0.9a'
 
 class Clip2ZeusApp(object):
 
-    EXPOSED = ('help', 'set_interval', 'shorten_urls', 'quit')
+    EXPOSED = ('help', 'get_interval', 'set_interval', 'shorten_urls', 'quit')
 
     def __init__(self, port=DEFAULT_PORT):
         """Creates the container for common functionality"""
 
+        logger.debug('Welcome to %s v%s' % (APP_TITLE, __version__))
+
         self.port = int(port)
 
         self.data = ''
+        self.server = None
         self._has_connection = False
         self.last_check = None
-        self.interval = 1
+        self.interval = config.get('main', 'interval', 1)
         self.threshold = timedelta(seconds=HEARTBEAT_INT)
         socket.setdefaulttimeout(TIMEOUT_SEC)
 
-        self.monitor_thread = Monitor(app=self)
+        self.thread_event = Event()
+        self.monitor_thread = Thread(target=self.monitor_clipboard)
 
     def expose_api(self):
         """Exposes a collection of commands for the XML-RPC server"""
 
+        logger.debug('Exposing API')
         for func in Clip2ZeusApp.EXPOSED:
+            logger.debug(' - %s' % func)
             self.server.register_function(getattr(self, func))
 
     def start(self):
         """Begins processing"""
 
+        logger.debug('Beginning Clip2Zeus')
         self.monitor_thread.start()
         self.start_server()
 
     def start_server(self):
         """Starts the XML-RPC server"""
 
+        if not port_is_free(self.port):
+            logger.error('Port %s is already being used.' % self.port)
+            self.quit()
+            sys.exit(1)
+
         try:
-            self.server = SimpleXMLRPCServer(('localhost', self.port), allow_none=True)
+            logger.debug('Starting server on port %s...' % self.port)
+            self.server = Server(('localhost', self.port), allow_none=True, logRequests=False)
             self.expose_api()
             self.server.serve_forever()
-        except socket.error, err:
-            # my sad excuse for a graceful termination
-            if err.errno == 9:
-                pass
-            elif err.errno == 48: # address already used
-                pass
-            else:
-                raise err
         except KeyboardInterrupt:
             self.quit()
 
@@ -149,6 +129,12 @@ class Clip2ZeusApp(object):
 
         pass
 
+    def get_interval(self, default=1):
+        """Returns the current polling interval in seconds"""
+
+        logger.debug('Interval requested by client: %s' % (self.interval,))
+        return self.interval
+
     def set_interval(self, interval):
         """Sets the clipboard polling frequency.
 
@@ -156,12 +142,32 @@ class Clip2ZeusApp(object):
         Use 0 to represent manual invocation."""
 
         try:
+            logger.debug('Updating interval to %s' % interval)
             self.interval = int(interval)
 
             if self.interval < 0:
                 raise ValueError
-        except (TypeError, ValueError):
-            raise TypeError('Please specify an integer that is 0 or greater.')
+
+            config.set('main', 'interval', self.interval)
+        except (TypeError, ValueError), err:
+            logger.error('%s' % (err,))
+            raise ValueError('Please specify an integer that is 0 or greater.')
+
+    def monitor_clipboard(self):
+        """Regularly checks the system clipboard for data"""
+
+        while True:
+            if self.interval <= 0:
+                wait = 1
+            else:
+                wait = self.interval
+
+            self.thread_event.wait(wait)
+            if self.thread_event.isSet():
+                break
+
+            if self.interval > 0:
+                self.shorten_urls()
 
     def shorten_urls(self):
         """Shortens any URLs that are currently in the clipboard"""
@@ -171,6 +177,9 @@ class Clip2ZeusApp(object):
             data = self.check_clipboard()
 
             if data and data != self.data:
+                logger.info('Found new clipboard data to process.')
+                logger.debug('Old data: %s' % (self.data, ))
+                logger.debug('New data: %s' % (data, ))
                 self.process_clipboard(data)
 
     def process_clipboard(self, data):
@@ -179,6 +188,8 @@ class Clip2ZeusApp(object):
         If one or more are found, they will be replaced with a 2zeus-shortened
         version of the longer URL."""
 
+        logger.info('Shortening URLs in clipboard')
+
         update_data = False
         matches = URL_RE.findall(data)
 
@@ -194,6 +205,7 @@ class Clip2ZeusApp(object):
                 })
 
                 try:
+                    logger.info(' - %s' % url)
                     c = urllib2.urlopen('http://2ze.us/generate/', params)
                 except (urllib2.HTTPError, urllib2.URLError):
                     # do something?
@@ -203,24 +215,30 @@ class Clip2ZeusApp(object):
                     update_data = True
                     raw = c.read()
                     try:
-                        json = simplejson.loads(raw)
+                        usable = json.loads(raw)
                     except (ValueError, ):
-                        json = None
+                        usable = None
 
-                if json:
-                    data = data.replace(url, json['urls'][url]['shortcut'])
+                if usable:
+                    short = usable['urls'][url]['shortcut']
+                    logger.info('   -> %s' % short)
+                    data = data.replace(url, short)
+
+        self.data = data
 
         if update_data:
-            self.data = data
+            logger.debug('Updating clipboard')
             self.update_clipboard(self.data)
 
     def quit(self):
         """Ends processing"""
 
-        print 'Exiting.'
-        self.monitor_thread.close()
-        self.server.server_close() # see if there's a better way to handle this
-        return 0
+        logger.info('Exiting.')
+        self.thread_event.set()
+        self.monitor_thread.join()
+
+        if self.server:
+            self.server.kill()
 
 class Clip2ZeusCtl(object):
     """
@@ -233,56 +251,47 @@ class Clip2ZeusCtl(object):
         self.proxy = None
         self.connect()
 
-    def launch_server(self):
-        """Launches the server if needed--not a good idea with the CLI command"""
-
-        try:
-            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-            sock.bind(('localhost', DEFAULT_PORT))
-        except socket.error, err:
-            if err.errno == 48: # address already in use
-                pass
-            else:
-                raise err
-        else:
-            sock.close()
-            from clip2zeus import main
-            self.server_thread = Thread(target=main)
-            self.server_thread.start()
-            time.sleep(1)
-
     def notify(self, message):
         """Tells the user something"""
 
+        logger.debug('Received message: %s' % (message, ))
         print message
 
     def execute_command(self, cmd, args=[]):
         """Calls a command using the XML-RPC proxy"""
 
+        logger.debug('Executing command "%s" via XML-RPC' % (cmd, ))
         try:
             if cmd == 'help':
                 self.help(*args)
             elif cmd in Clip2ZeusApp.EXPOSED:
                 func = getattr(self.proxy, cmd)
-                func(*args)
+                return func(*args)
             else:
+                logger.error('Invalid command: %s' % (cmd, ))
                 sys.exit('Invalid command.  Options include: %s' % ', '.join(Clip2ZeusApp.EXPOSED))
         except socket.error, err:
-            self.notify('Failed to connect to application: %s' % err)
+            self.notify('Failed to connect to application: %s' % (err[1], ))
 
-            if err.errno == 61:
+            if err[0] in (48, 61, 10061):
                 sys.exit(1)
         except xmlrpclib.Fault, fault:
-            self.notify('%s' % fault)
+            self.notify('%s' % (fault, ))
 
-    def connect(self, port=None):
+    def connect(self, port=DEFAULT_PORT):
         """Attempt to connect to the XML-RPC server"""
 
+        logger.debug('Connecting to XML-RPC server')
+        if port_is_free(port):
+            logger.error('Port is not in use')
+            sys.exit(1)
+
         try:
             port = port or self.port
             self.proxy = xmlrpclib.ServerProxy('http://localhost:%s/' % port, allow_none=True)
         except Exception, e:
-            print e
+            logger.error('%s' % (e, ))
+            raise e
         else:
             self.port = port
 
